from emodpy_hiv.campaign.common import TargetDemographicsConfig, PropertyRestrictions, TargetGender, RepetitionConfig
from emodpy_hiv.campaign.distributor import add_intervention_scheduled, add_intervention_triggered
from emodpy_hiv.campaign.individual_intervention import BroadcastEvent, SimpleVaccine
from emodpy_hiv.campaign.waning_config import MapLinear
from emodpy_hiv.utils.distributions import UniformDistribution, ExponentialDistribution
from emodpy_hiv.utils.targeting_config import IsPregnant


def build_campaign(campaign):
    # Define a list of interventions to be scheduled, which will be wrapped in a MultiInterventionDistributor
    interventions = [BroadcastEvent(campaign, "Test_Event"),
                     SimpleVaccine(campaign, waning_config=MapLinear([2010, 2020], [0.9, 0.95]))]

    # Define a uniform distribution for delay, which will be used in a DelayedIntervention.
    uniform_distribution = UniformDistribution(0, 365)

    # Add a scheduled event: This will use the StandardInterventionDistributionEventCoordinator
    add_intervention_scheduled(campaign=campaign,
                               intervention_list=interventions,
                               start_year=1990,  # Start year for the intervention
                               event_name="test_event",  # Name of the event
                               node_ids=[1, 2, 3],  # Apply the event to specific nodes
                               target_demographics_config=TargetDemographicsConfig(
                                   demographic_coverage=0.7,  # Target 70% of the population
                                   target_gender=TargetGender.FEMALE),  # Target only female individuals
                               repetition_config=RepetitionConfig(
                                   number_repetitions=2,  # Repeat the event twice
                                   timesteps_between_repetitions=365),  # Interval between repetitions
                               property_restrictions=PropertyRestrictions(
                                   individual_property_restrictions=[["Risk:High"]]),
                               # Restrict to individuals with "Risk:High"
                               delay_distribution=uniform_distribution,  # Apply delay using uniform distribution
                               targeting_config=~IsPregnant())  # Exclude pregnant individuals

    # Define a list of interventions to be scheduled, which will be wrapped in a MultiInterventionDistributor
    interventions = [BroadcastEvent(campaign, "Test_BroadcastEvent"),
                     SimpleVaccine(campaign, waning_config=MapLinear([2010, 2020], [0.9, 0.95]))]

    # Define an exponential distribution for delay, which will be used in a DelayedIntervention.
    exponential_distribution = ExponentialDistribution(10)

    # Add a triggered event: This will use the StandardInterventionDistributionEventCoordinator with NodeLevelHealthTriggeredIV
    add_intervention_triggered(campaign=campaign,
                               intervention_list=interventions,
                               triggers_list=["Trigger1", "Trigger2"],  # List of triggers for the intervention
                               event_name="test_event",  # Name of the event
                               node_ids=[1, 2, 3],  # Apply the event to specific nodes
                               start_year=1990,  # Start year for the intervention
                               duration=365,  # Duration of the intervention
                               target_demographics_config=TargetDemographicsConfig(
                                   demographic_coverage=0.7,  # Target 70% of the population
                                   target_gender=TargetGender.MALE,  # Target only male individuals
                                   target_age_min=10,  # Minimum age of targeted individuals
                                   target_age_max=20),  # Maximum age of targeted individuals
                               property_restrictions=PropertyRestrictions(
                                   individual_property_restrictions=[["Risk:High"]]),
                               # Restrict to individuals with "Risk:High"
                               delay_distribution=exponential_distribution,
                               # Apply delay using exponential distribution
                               targeting_config=~IsPregnant())  # Exclude pregnant individuals
    return campaign


if __name__ == "__main__":
    # Example usage of the build_campaign function
    import emod_api.campaign as campaign_api

    # Set the schema for the campaign
    campaign_api.set_schema("schema.json")
    # Build the campaign
    campaign_api = build_campaign(campaign_api)
    # Save the campaign to a JSON file
    campaign_api.save("distribute_multiple_interventions.json")
