from typing import List, Tuple, Any
import os
import shutil

from langchain_community.vectorstores import FAISS as VectorStore
from langchain_community.retrievers import BM25Retriever
from langchain_openai import OpenAIEmbeddings, ChatOpenAI
from langchain.chains import RetrievalQA
from langchain.schema import Document


def perform_bm25_retrieval(
    docs: List[Document], split_docs: List[Document], query: str, num_docs: int
) -> List[Document]:
    """
    Retrieve documents using BM25 keyword matching.
    """
    all_docs = docs + split_docs
    print("Retrieval using Keyword Matching Algorithm.")
    bm25_retriever = BM25Retriever.from_documents(all_docs)
    bm25_retriever.k = 2 * num_docs if num_docs < 10 else num_docs + 10
    return bm25_retriever.invoke(query)


def perform_dense_retrieval(
    retrieved_docs: List[Document],
    key_docs: List[Document],
    chroma_db_path: str,
    openai_api_key: str,
    num_docs: int,
    embedding_model: str = "text-embedding-3-small"
) -> Tuple[Any, Any]:
    """
    Retrieve documents using dense semantic search (FAISS + OpenAI embeddings).
    """
    all_docs = retrieved_docs + key_docs
    embedding = OpenAIEmbeddings(model=embedding_model, openai_api_key=openai_api_key)
    print("Retrieval using Chroma semantic search.")
    for doc in all_docs:
        if not isinstance(doc, Document):
            raise ValueError("The retrieved documents must be of type Document.")
    if os.path.exists(chroma_db_path):
        shutil.rmtree(chroma_db_path)
    vector_db = VectorStore.from_documents(all_docs, embedding)
    dense_retriever = vector_db.as_retriever(search_kwargs={'k': num_docs})
    return dense_retriever, vector_db


def initialize_llm_and_qa_chain(
    dense_retriever: Any, openai_api_key: str, model: str
) -> Tuple[Any, Any]:
    """
    Initialize the LLM and QA chain.
    """
    llm = ChatOpenAI(model=model, openai_api_key=openai_api_key)
    qa_chain = RetrievalQA.from_chain_type(
        llm=llm,
        chain_type="stuff",
        retriever=dense_retriever,
        return_source_documents=True
    )
    return llm, qa_chain


def print_results(response: dict) -> None:
    """
    Print the RAG answer and metadata of retrieved sources.
    """
    print("\nRAG Answer:\n", response["result"])
    print("\nNumber of Retrieved Sources:", len(response["source_documents"]))
    print("\nRetrieved Sources Metadata:")
    for i, doc in enumerate(response["source_documents"], start=1):
        source = doc.metadata.get("source", "Unknown source")
        distributors = doc.metadata.get("distributors")
        interventions = doc.metadata.get("interventions")
        chunk_id = doc.metadata.get("chunk_id")
        chunk_size = doc.metadata.get("chunk_size")
        chunk_head = doc.metadata.get("chunk_head")
        print(f"Text Chunk {i}:")
        print(f"   - source: {source}")
        if chunk_id is not None:
            print(f"   - chunk_id: {chunk_id}")
        if chunk_size is not None:
            print(f"   - chunk_size: {chunk_size}")
        if chunk_head is not None:
            print(f"   - chunk_head: {chunk_head}")
        if interventions is not None:
            print(f"   - interventions: {interventions}")
        if distributors is not None:
            print(f"   - distributors: {distributors}")


def reset_chroma_db(chroma_db_fusion: Any) -> None:
    """
    Reset the Chroma database.
    """
    from chromadb.api.client import SharedSystemClient
    SharedSystemClient.clear_system_cache()
    all_ids = chroma_db_fusion.get()["ids"]
    chroma_db_fusion._collection.delete(ids=all_ids)
    chroma_db_fusion.delete_collection()


def generate_retriever(
    chroma_db_path: str,
    docs: List[Document],
    key_docs: List[Document],
    num_docs: int,
    openai_api_key: str,
    query: str,
    split_docs: List[Document]
) -> Tuple[Any, Any]:
    """
    Generate a retriever using BM25 and dense retrieval.
    """
    if not openai_api_key:
        raise ValueError("OpenAI API key is required.")
    if not docs:
        raise ValueError("No documents provided for retrieval.")
    retrieved_docs = perform_bm25_retrieval(docs, split_docs, query, num_docs)
    dense_retriever, vector_db = perform_dense_retrieval(
        retrieved_docs, key_docs, chroma_db_path, openai_api_key, num_docs
    )
    return vector_db, dense_retriever


def ask_llm(
    openai_api_key: str,
    query: str,
    dense_retriever: Any,
    vector_db: Any,
    model: str
) -> Tuple[dict, Any, Any]:
    """
    Query the LLM using the QA chain and print results.
    """
    if not openai_api_key:
        raise ValueError("OpenAI API key is required. Please provide it in the environment variable 'OPENAI_API_KEY' or "
                         "set it in the 'key.env' file as: OPENAI_API_KEY='your_openai_key_here'.")
    llm, qa_chain = initialize_llm_and_qa_chain(dense_retriever, openai_api_key, model=model)
    response = qa_chain.invoke({"query": query})
    print_results(response)
    if not isinstance(vector_db, VectorStore):
        reset_chroma_db(vector_db)
    return response, llm, qa_chain

