# python
import json
from pathlib import Path
from typing import List, Union

from langchain_community.document_loaders import JSONLoader, PythonLoader
from langchain_text_splitters import RecursiveCharacterTextSplitter, Language
from langchain.schema import Document
from bs4 import BeautifulSoup


def load_retrieval_documents(
        docs_path: Union[str, Path] = "./Docs",
        key_docs_path: Union[str, Path] = "./Docs/key_docs",
        split_docs_path: Union[str, Path] = "./Docs/split_docs",
) -> (List[Document], List[Document], List[Document]):
    """
    Load retrieval documents from provided folder paths.

    Splits documents in `split_docs_path` into large overlapping chunks.
    """
    docs = _load_all_docs(docs_path, "example_docs")
    key_docs = _load_all_docs(key_docs_path, "key_docs")
    split_docs = _load_all_docs(split_docs_path, "split_docs")

    splitter = RecursiveCharacterTextSplitter.from_language(
        language=Language.PYTHON, chunk_size=10000, chunk_overlap=200
    )
    split_docs = splitter.split_documents(split_docs)

    for i, doc in enumerate(split_docs):
        doc.metadata["chunk_id"] = i + 1
        doc.metadata["chunk_size"] = len(doc.page_content)
        doc.metadata["chunk_head"] = doc.page_content[:100].rstrip()
    _print_doc_summary(split_docs, "extra docs that are split into 10000-character chunks with 200-character overlap")

    return docs, key_docs, split_docs


def _load_all_docs(path: Union[str, Path, List], doc_type: str) -> List[Document]:
    """
    Load documents from a path or a list of uploaded files.

    Accepts JSON, Python, and HTML files.
    """
    if isinstance(path, list):
        json_files = [f for f in path if f.name.endswith(".json")]
        python_files = [f for f in path if f.name.endswith(".py")]
        html_files = [f for f in path if f.name.endswith(".html")]
    else:
        directory = Path(path)
        if not directory.exists():
            raise ValueError(f"The provided path {directory} is neither a file nor a directory.")
        json_files = list(directory.glob("*.json"))
        python_files = list(directory.glob("*.py"))
        html_files = list(directory.glob("*.html"))
    docs = load_doc(json_files, python_files, html_files)
    _print_doc_summary(docs, doc_type)
    return docs


def json_metadata_func(record: dict, metadata: dict) -> dict:
    """
    Extract metadata for JSON documents.

    Adds 'distributors' and 'interventions' to the metadata.
    """
    distributors = {
        record["Events"][i]["Event_Coordinator_Config"].get("class")
        for i in range(len(record["Events"]))
        if record["Events"][i]["Event_Coordinator_Config"].get("class") is not None
    }
    metadata["distributors"] = str(distributors)

    interventions = {
        record["Events"][i]["Event_Coordinator_Config"]["Intervention_Config"].get("class")
        for i in range(len(record["Events"]))
        if record["Events"][i]["Event_Coordinator_Config"]["Intervention_Config"].get("class") is not None
    }
    metadata["interventions"] = str(interventions)
    return metadata


def load_doc(
        json_files: List[Union[str, Path]],
        python_files: List[Union[str, Path]],
        html_files: List[Union[str, Path]]
) -> List[Document]:
    """
    Load documents from JSON, Python, and HTML file lists.
    """
    docs: List[Document] = []
    for json_file in json_files:
        loader = JSONLoader(
            str(json_file), jq_schema=".", content_key="Events", text_content=False, metadata_func=json_metadata_func
        )
        docs.extend(loader.load())
    for python_file in python_files:
        loader = PythonLoader(str(python_file))
        docs.extend(loader.load())
    for html_file in html_files:
        docs.append(_load_html(Path(html_file)))
    return docs


def _load_html(html_file: Path) -> Document:
    """
    Load and parse an HTML file into a Document.
    """
    with html_file.open("r", encoding="utf-8") as f:
        html_content = f.read()
    soup = BeautifulSoup(html_content, "html.parser")
    text = soup.get_text()
    return Document(page_content=text, metadata={"source": str(html_file), "type": "html",
                                                 "description": f"{html_file.name} documentation"})


def _print_doc_summary(docs: List[Document], doc_type: str = "key_docs") -> None:
    """
    Print a summary of a list of Documents.
    """
    print(f"Total number of {doc_type} pages: {len(docs)}")
    for i, doc in enumerate(docs):
        print(f"Page {i + 1} length: {len(doc.page_content)} characters")
    if docs:
        print(f"Metadata for the first page is:\n{json.dumps(docs[0].metadata, indent=4)}")