import json
import re
import os

python_template = """
```python
# import modules

def build_campaign(campaign):
    # variables declaration

    # For each event in the expected JSON:
        # step 1, create intervention instance(s) from class(es) imported from `emodpy_hiv.campaign.individual_intervention` or `emodpy_hiv.campaign.node_intervention`.
        # step 2, distribute the intervention instance(s) using distributor helper functions from `emodpy_hiv.campaign.distributor`.
        # Use the `add_intervention_triggered` function to add the intervention with `Trigger_Condition_List` and `NodeLevelHealthTriggeredIV`.
        # Use loop to create multiple events with different parameters when possible.
    return campaign
```
"""


def _get_query(json, python=python_template):
    """
    Generates a query for the LLM based on the expected JSON structure.
    """
    query = f"""
Please write a Python script that generates the following JSON using the emodpy package.

Expected JSON:
{json}

Format your response using this template:
{python}

Please follow these guidelines:

- Enclose the entire Python script between "```python" and "```" tags.
- Use the numerical values from the expected JSON (do not reuse values from any input JSON).
- Include complete code for all events in the JSON, with all necessary import statements.
- The script should define a function `build_campaign(campaign)` that takes a campaign object and returns the modified campaign object.
- Use the appropriate intervention classes from `emodpy_hiv.campaign.individual_intervention` or `emodpy_hiv.campaign.node_intervention`.
- Use distributor helper functions from `emodpy_hiv.campaign.distributor`.
- Refer to the HTML documentation of the emodpy package to find the correct classes and methods.
- If no relevant Python or HTML examples are found based on the expected JSON, reply with a message indicating that no example is available.
"""

    # - Use classes from `emodpy_hiv.campaign.common` where applicable.
    return query


def load_query(json_path):
    """
    Loads a JSON file or string and generates a query for the LLM.
    """
    if isinstance(json_path, str) or isinstance(json_path, os.PathLike):
        with open(json_path, 'r') as file:
            json_data = json.load(file)
    # It could be a UploadedFile object
    else:
        json_data = json.load(json_path)
    return _get_query(json_data)


def extract_and_save_code(response: str, output_filename: str) -> None:
    """
    Extracts Python code enclosed between ```python and ```endOfCode markers
    from the response and saves it to a specified Python file.

    Parameters:
    - response (str): The text containing the Python code.
    - output_filename (str): The name of the file to save the extracted code.

    Returns:
    - None
    """
    # Regular expression pattern to match content between ```python and ```endOfCode
    pattern = r"```python\s+(.*?)```"

    # Search for the pattern in the response
    match = re.search(pattern, response, re.DOTALL)

    if match:
        code_content = match.group(1).strip()
        with open(output_filename, 'w', encoding='utf-8') as file:
            file.write(code_content + '\n')
    else:
        print("Warning: No code block found between the specified markers.")