import streamlit as st
import tempfile
import os
from langchain_community.document_loaders import (
    JSONLoader, PythonLoader, PyPDFLoader, TextLoader, UnstructuredWordDocumentLoader
)
from langchain_text_splitters import RecursiveCharacterTextSplitter, Language
from emodpy_converter.template import extract_and_save_code, load_query
from emodpy_converter.doc_loader import json_metadata_func, _load_html
from emodpy_converter.ask_llm import ask_llm, generate_retriever
from emodpy_converter.secrets_loader import get_openai_api_key


# --- UI Setup ---
st.title("🔗EMOD Campaign JSON to Emodpy Script Converter.")

openai_api_key = st.sidebar.text_input("Please enter your OpenAI API key", type="password")
#if not openai_api_key:
#    st.sidebar.warning("Please enter your OpenAI API key to use this tool.")
openai_api_key = get_openai_api_key(openai_api_key)
models = ["o3-mini-2025-01-31", "gpt-4o-2024-08-06", "gpt-4.1-2025-04-14"]
model = st.sidebar.selectbox("Select the model to use", models, index=0)
st.sidebar.write('You selected model:', model)

st.markdown("Upload campaign json file (json) and internal documents (PDF, HTML or Python script) to analyze using AI.")

# --- File Uploads ---
json_file = st.file_uploader("Upload json files that you want to convert to Emodpy script", type=["json"])
doc_files = st.file_uploader("Upload documents (PDF, HTML) and example python script (.py)", type=["pdf", "html", "py"], accept_multiple_files=True)
key_doc_files = st.file_uploader("Optional: Upload key documents (PDF, HTML) and example python script (.py); these files will not be filtered out by RAG.", type=["pdf", "html", "py"], accept_multiple_files=True)
split_doc_files = st.file_uploader("Optional: Upload optional documents (PDF, HTML) and example python script (.py); these files will be split and filtered by RAG.", type=["pdf", "html", "py"], accept_multiple_files=True)
optional_query = st.text_input("Optional: Enter additional keywords to help AI understand the context of the json file. For example, 'ControlledVaccine'")
submit = st.button("Generate Emodpy script")


# --- Document Processing ---
def load_document(file, suffix):
    with tempfile.NamedTemporaryFile(delete=False, suffix="." + suffix) as tmp_file:
        tmp_file.write(file.read())
        tmp_file.flush()
        if suffix == "pdf":
            return PyPDFLoader(tmp_file.name).load()
        elif suffix == "html":
            from pathlib import Path
            return [_load_html(Path(tmp_file.name))]
        elif suffix == "py":
            return PythonLoader(tmp_file.name).load()
        elif suffix == "json":
            return JSONLoader(tmp_file.name, jq_schema='.', content_key="Events", text_content=False, metadata_func=json_metadata_func).load()
        elif suffix == "txt":
            return TextLoader(tmp_file.name).load()
        elif suffix == "docx":
            return UnstructuredWordDocumentLoader(tmp_file.name).load()
        return []


def process_documents(files):
    docs = []
    for f in files:
        suffix = f.name.split(".")[-1].lower()
        try:
            docs.extend(load_document(f, suffix))
        except Exception as e:
            st.warning(f"⚠️ Could not read {f.name}: {str(e)}")
    return docs


def split_documents(docs):
    splitter = RecursiveCharacterTextSplitter.from_language(
        language=Language.PYTHON,
        chunk_size=10000,
        chunk_overlap=200
    )
    split_docs = splitter.split_documents(docs)
    for i, doc in enumerate(split_docs):
        doc.metadata["chunk_id"] = i + 1
        doc.metadata["chunk_size"] = len(doc.page_content)
        doc.metadata["chunk_head"] = doc.page_content[:100].rstrip()
    return split_docs


# --- Query Preparation ---
query = None
if not openai_api_key:
    st.warning("Please enter your OpenAI API key to use this tool.")
if json_file:
    if isinstance(json_file, list):
        st.warning("Multiple json files detected. Using the first one.")
        json_file = json_file[0]
    query = load_query(json_file)
    if optional_query:
        query += f"\n\nKeywords: {optional_query}"
else:
    st.warning("Please upload a json file to convert to Emodpy script.")
if not doc_files:
    st.warning("Please upload at least one document (PDF, HTML, or Python script) to analyze.")

# --- Main Processing ---
if submit:
    raw_docs = process_documents(doc_files) if doc_files else []
    if doc_files and not raw_docs:
        st.warning("No valid documents could be processed.")
    if doc_files:
        st.subheader("📄 Docs")

    key_docs = process_documents(key_doc_files) if key_doc_files else []
    if key_doc_files:
        st.subheader("📄 Key Docs")

    split_docs = process_documents(split_doc_files) if split_doc_files else []
    if split_doc_files:
        if split_docs:
            split_docs = split_documents(split_docs)
            st.subheader("📄 Split Docs")
        else:
            st.warning("No valid split documents could be processed.")

    if json_file and openai_api_key and raw_docs:
        st.subheader("📊 AI Response")
        with st.spinner("Ask LLM to convert json to Emodpy script..."):
            chroma_db_path = './chroma_db'
            num_docs = 12
            chroma_db_fusion, dense_retriever = generate_retriever(
                chroma_db_path, raw_docs, key_docs, num_docs, openai_api_key, query, split_docs
            )
            response, llm, qa_chain = ask_llm(openai_api_key, query, dense_retriever, chroma_db_fusion, model=model)
            output_folder = "./output"
            os.makedirs(output_folder, exist_ok=True)
            output_filename = json_file.name.replace(".json", ".py")
            extract_and_save_code(response["result"], os.path.join(output_folder, output_filename))
        st.write(response["result"])
        st.subheader("Original Json")
        json_content = json_file.getvalue().decode("utf-8")
        st.text_area("Original Json File Content", json_content, height=300)


